@@ -3,7 +3,7 @@ import typing
 
 from starlette.requests import Request
 from starlette.types import Receive, Scope, Send
-from starlette.websockets import WebSocket, WebSocketState
+from starlette.websockets import WebSocket, WebSocketDisconnect, WebSocketState
 
 from graphql import GraphQLError
 from graphql.error import format_error as format_graphql_error
@@ -39,7 +39,7 @@ class GraphQL:
         self.graphiql = graphiql
         self.keep_alive = keep_alive
         self.keep_alive_interval = keep_alive_interval
-        self._keep_alive_task = None
+        self._keep_alive_task: typing.Optional[asyncio.Task[typing.Any]] = None
         self.debug = debug
 
     async def __call__(self, scope: Scope, receive: Receive, send: Send):
@@ -47,7 +47,7 @@ class GraphQL:
             await self.handle_http(scope=scope, receive=receive, send=send)
         elif scope["type"] == "websocket":
             await self.handle_websocket(scope=scope, receive=receive, send=send)
-        else:
+        else:  # pragma: no cover
             raise ValueError("Unknown scope type: %r" % (scope["type"],))
 
     async def get_root_value(self, request: Request) -> typing.Optional[typing.Any]:
@@ -59,38 +59,70 @@ class GraphQL:
         return {"request": request}
 
     async def handle_keep_alive(self, websocket):
-        if websocket.application_state == WebSocketState.DISCONNECTED:
+        if (
+            websocket.application_state == WebSocketState.DISCONNECTED
+        ):  # pragma: no cover
             return
 
-        await websocket.send_json({"type": GQL_CONNECTION_KEEP_ALIVE})
         await asyncio.sleep(self.keep_alive_interval)
+        await websocket.send_json({"type": GQL_CONNECTION_KEEP_ALIVE})
 
         self._keep_alive_task = asyncio.create_task(self.handle_keep_alive(websocket))
 
     async def handle_websocket(self, scope: Scope, receive: Receive, send: Send):
         websocket = WebSocket(scope=scope, receive=receive, send=send)
 
+        subscriptions: typing.Dict[str, typing.AsyncGenerator] = {}
+        tasks = {}
+
         await websocket.accept(subprotocol="graphql-ws")
 
-        while websocket.application_state != WebSocketState.DISCONNECTED:
-            message = await websocket.receive_json()
+        try:
+            while (
+                websocket.client_state != WebSocketState.DISCONNECTED
+                and websocket.application_state != WebSocketState.DISCONNECTED
+            ):
+                message = await websocket.receive_json()
 
-            operation_id = message.get("id")
-            message_type = message.get("type")
+                operation_id = message.get("id")
+                message_type = message.get("type")
 
-            if message_type == GQL_CONNECTION_INIT:
-                await websocket.send_json({"type": GQL_CONNECTION_ACK})
+                if message_type == GQL_CONNECTION_INIT:
+                    await websocket.send_json({"type": GQL_CONNECTION_ACK})
 
-                if self.keep_alive:
-                    asyncio.create_task(self.handle_keep_alive(websocket))
-            elif message_type == GQL_CONNECTION_TERMINATE:
-                await websocket.close()
-            elif message_type == GQL_START:
-                await self.start_subscription(
-                    message.get("payload"), operation_id, websocket
-                )
-            elif message_type == GQL_STOP:
-                await websocket.close()
+                    if self.keep_alive:
+                        self._keep_alive_task = asyncio.create_task(
+                            self.handle_keep_alive(websocket)
+                        )
+                elif message_type == GQL_CONNECTION_TERMINATE:
+                    await websocket.close()
+                elif message_type == GQL_START:
+                    async_result = await self.start_subscription(
+                        message.get("payload"), operation_id, websocket
+                    )
+
+                    subscriptions[operation_id] = async_result
+
+                    tasks[operation_id] = asyncio.create_task(
+                        self.handle_async_results(async_result, operation_id, websocket)
+                    )
+                elif message_type == GQL_STOP:  # pragma: no cover
+                    if operation_id not in subscriptions:
+                        return
+
+                    await subscriptions[operation_id].aclose()
+                    tasks[operation_id].cancel()
+                    del tasks[operation_id]
+                    del subscriptions[operation_id]
+        except WebSocketDisconnect:  # pragma: no cover
+            pass
+        finally:
+            if self._keep_alive_task:
+                self._keep_alive_task.cancel()
+
+            for operation_id in subscriptions:
+                await subscriptions[operation_id].aclose()
+                tasks[operation_id].cancel()
 
     async def start_subscription(self, data, operation_id: str, websocket: WebSocket):
         query = data["query"]
@@ -102,21 +134,25 @@ class GraphQL:
 
         context = await self.get_context(websocket)
 
-        data = await self.schema.subscribe(
+        return await self.schema.subscribe(
             query,
             variable_values=variables,
             operation_name=operation_name,
             context_value=context,
         )
 
+    async def handle_async_results(
+        self, results: typing.AsyncGenerator, operation_id: str, websocket: WebSocket
+    ):
         try:
-            async for result in data:
+            async for result in results:
                 payload = {"data": result.data}
 
                 if result.errors:
                     payload["errors"] = [
                         format_graphql_error(err) for err in result.errors
                     ]
+
                 await self._send_message(websocket, GQL_DATA, payload, operation_id)
         except Exception as error:
             if not isinstance(error, GraphQLError):
@@ -129,12 +165,11 @@ class GraphQL:
                 operation_id,
             )
 
-        await self._send_message(websocket, GQL_COMPLETE, None, operation_id)
-
-        if self._keep_alive_task:
-            self._keep_alive_task.cancel()
-
-        await websocket.close()
+        if (
+            websocket.client_state != WebSocketState.DISCONNECTED
+            and websocket.application_state != WebSocketState.DISCONNECTED
+        ):
+            await self._send_message(websocket, GQL_COMPLETE, None, operation_id)
 
     async def _send_message(
         self,
