@@ -1,12 +1,19 @@
 from typing import Any, Dict, List, Optional, Sequence, Type, Union
 
-from graphql import GraphQLSchema, get_introspection_query, parse, validate_schema
+from graphql import (
+    ExecutionContext as GraphQLExecutionContext,
+    GraphQLSchema,
+    get_introspection_query,
+    parse,
+    validate_schema,
+)
 from graphql.subscription import subscribe
 from graphql.type.directives import specified_directives
 
 from strawberry.custom_scalar import ScalarDefinition
 from strawberry.enum import EnumDefinition
 from strawberry.extensions import Extension
+from strawberry.schema.schema_converter import GraphQLCoreConverter
 from strawberry.types.types import TypeDefinition
 from strawberry.union import StrawberryUnion
 
@@ -14,7 +21,6 @@ from ..middleware import DirectivesMiddleware, Middleware
 from ..printer import print_schema
 from .base import ExecutionResult
 from .execute import execute, execute_sync
-from .types import ConcreteType, get_directive_type, get_object_type
 
 
 class Schema:
@@ -27,20 +33,29 @@ class Schema:
         directives=(),
         types=(),
         extensions: Sequence[Type[Extension]] = (),
+        execution_context_class: Optional[Type[GraphQLExecutionContext]] = None,
     ):
         self.extensions = extensions
-        self.type_map: Dict[str, ConcreteType] = {}
 
-        query_type = get_object_type(query, self.type_map)
-        mutation_type = get_object_type(mutation, self.type_map) if mutation else None
+        self.execution_context_class = execution_context_class
+
+        self.schema_converter = GraphQLCoreConverter()
+
+        query_type = self.schema_converter.from_object_type(query)
+        mutation_type = (
+            self.schema_converter.from_object_type(mutation) if mutation else None
+        )
         subscription_type = (
-            get_object_type(subscription, self.type_map) if subscription else None
+            self.schema_converter.from_object_type(subscription)
+            if subscription
+            else None
         )
 
         self.middleware: List[Middleware] = [DirectivesMiddleware(directives)]
 
         directives = [
-            get_directive_type(directive, self.type_map) for directive in directives
+            self.schema_converter.from_directive(directive.directive_definition)
+            for directive in directives
         ]
 
         self._schema = GraphQLSchema(
@@ -48,7 +63,7 @@ class Schema:
             mutation=mutation_type,
             subscription=subscription_type if subscription else None,
             directives=specified_directives + directives,
-            types=[get_object_type(type, self.type_map) for type in types],
+            types=list(map(self.schema_converter.from_object_type, types)),
         )
 
         # Validate schema early because we want developers to know about
@@ -58,15 +73,15 @@ class Schema:
             formatted_errors = "\n\n".join(f"❌ {error.message}" for error in errors)
             raise ValueError(f"Invalid Schema. Errors:\n\n{formatted_errors}")
 
-        self.query = self.type_map[query_type.name]
+        self.query = self.schema_converter.type_map[query_type.name]
 
     def get_type_by_name(
         self, name: str
     ) -> Optional[
         Union[TypeDefinition, ScalarDefinition, EnumDefinition, StrawberryUnion]
     ]:
-        if name in self.type_map:
-            return self.type_map[name].definition
+        if name in self.schema_converter.type_map:
+            return self.schema_converter.type_map[name].definition
 
         return None
 
@@ -87,6 +102,7 @@ class Schema:
             operation_name=operation_name,
             additional_middlewares=self.middleware,
             extensions=self.extensions,
+            execution_context_class=self.execution_context_class,
         )
 
         return ExecutionResult(
@@ -112,6 +128,7 @@ class Schema:
             operation_name=operation_name,
             additional_middlewares=self.middleware,
             extensions=self.extensions,
+            execution_context_class=self.execution_context_class,
         )
 
         return ExecutionResult(
